# Colormaps for Swan

#from numpy import *
from pylab import *
import matplotlib as mpl


def hsv2rgb(ctuple):
    """Convert hsv tuple to rgb tuple
    based on wikipedia page"""
    h,s,v = ctuple
    h = (h*360.0)%360. # hue is an angle

    hi = floor(h/60.)

    f = h/60. - hi
    if not hi%2 : f= 1-f
    p = v*(1 - s)
    q = v*(1 - s*f)
    
    if (hi == 0): r,g,b = v,q,p
    elif (hi == 1): r,g,b = q,v,p
    elif (hi == 2): r,g,b = p,v,q
    elif (hi == 3): r,g,b = p,q,v
    elif (hi == 4): r,g,b = q,p,v
    elif (hi == 5): r,g,b = v,p,q
    
    return r,g,b


def get_rgbswan_data2():
    sat,val = 0.99,0.99
    colors = {'red':0,'green':1,'blue':2}
    step = 0.05
    nsteps = 256

    #hues = linspace(0.2,1, nsteps)
    hues = [0.7, 0.6, 0.5, 0.45, 0.3, 0.2, 0.15, 0.1, 0.0]
    hue_ind = linspace(0, 1, len(hues))

    rgb_vect = [ (i, hsv2rgb((hue, sat, val))) 
                 for i,hue in zip(hue_ind, hues)]
    res = {}

    for color,ind in colors.items():
        res[color] = [(x[0], x[1][ind], x[1][ind]) for x in rgb_vect]    
    return res

def get_rgswan_data():
    colors = {'red':0,'green':1,'blue':2}
    nhues = 25
    hue_ind = linspace(0.0, 1.0, 2*nhues)
    reds,greens = np.ones(2*nhues), np.ones(2*nhues)
    reds[:nhues] = linspace(0,1.0,nhues)
    greens[nhues:] = linspace(0,1.0,nhues)[::-1]
    rgb_vect = [(i, (r, g, 0.0)) for i,r,g in zip(hue_ind,reds,greens)]
    res = {}
    for color,ind in colors.items():
	res[color] = [(x[0], x[1][ind], x[1][ind]) for x in rgb_vect]
    return res

def set_rgbswan_cm():
    LUTSIZE = mpl.rcParams['image.lut']
    
    _rgbswan_data =  get_rgbswan_data2() 

    cm.datad['rgbswan'] = _rgbswan_data
    cm.rgbswan  = mpl.colors.LinearSegmentedColormap('rgbswan',
                                                            _rgbswan_data, LUTSIZE)
    return cm.rgbswan

def _make_swanrg():
    LUTSIZE = mpl.rcParams['image.lut']
    _rgswan_data =  get_rgswan_data()
    cmap = mpl.colors.LinearSegmentedColormap('rgswan',
					      _rgswan_data, LUTSIZE)
    return cmap

def _make_swanrgb():
    LUTSIZE = mpl.rcParams['image.lut']
    _rgbswan_data =  get_rgbswan_data2()
    cmap = mpl.colors.LinearSegmentedColormap('rgbswan',
					      _rgbswan_data, LUTSIZE)
    return cmap

swanrgb = _make_swanrgb()
swansummer =  _make_swanrg()


def _swanrgb():
    '''
    set the default colormap to rgbswan and apply to current image if any.
    See help(colormaps) for more information
    '''
    rc('image', cmap='rgbswan')
    im = gci()

    if im is not None:
        im.set_cmap(cm.rgbswan)
    draw_if_interactive()



# General purpose utilities

import numpy as np
from numpy import log, arange, ones
import pylab as pl

import swan.gui
from swan import pycwt
swanrgb = swan.gui.swanrgb


def log2(x):
    """Base 2 logarithm"""
    return log(x)/log(2.)

def make_progress_str(i,L=100):
    """Make a progress string with a rotating stick"""
    return '\r%3d%s %s' % (100*i/L, '%', ('|','/','-','\\')[i%4])


def deriv(x,y):
    """Calculate 3-point derivative of a signal"""
    return  .5*((y[1:-1] - y[0:-2]) / (x[1:-1] - x[0:-2]) + \
		(y[2:] - y[1:-1]) / (x[2:] - x[1:-1]))

def get_extr_deriv(x,y,maxf=True,wth=4):
    """Derivative-based extrema detection"""
    der = deriv(x,y)
    a = der > 0
    if maxf:
        a = a[1:] < a[:-1]
    else:
        a = a[1:] > a[:-1]

    k = np.arange(len(a))[a]
    xe = np.ones(len(k),dtype='d')
    ye = np.ones(len(k),dtype='d')

    for j,i in enumerate(k):
        p = np.polyfit(x[i+1:i+3], der[i:i+2], 1)
        xe[j] = -p[1] / p[0] #???
        p = np.polyfit(x[i:i+3], y[i:i+3], 2)
        #ye[j] = polyval(p, x[i+1:i+2])[0]
        ye[j] = np.polyval(p, (xe[j],) )[0]
    return xe,ye

def detrend1(vect, n=1):
    j = arange(len(vect))
    return vect - polyval(polyfit(j, vect, n), j)

def deoutburst(vect, n = 4):
    d = np.std(vect)
    m = np.mean(vect)

    low = m - n*d
    high = m+ n*d

    res = vect.copy()
    for i in xrange(len(vect)):
        if vect[i] < low:
            res[i] = low
        elif vect[i] > high:
            res[i] = high
        else:
            res[i] = vect[i]
    return res

def default_freqs(Ns, f_s, num=100):
    """
    Return default frequencies vector
    -- Ns:  number of samples in data vector
    -- f_s: sampling frequency
    -- num: number of frequencies required
    """
    T = Ns/f_s
    return np.linspace(4/T, f_s/2, num=num)

def default_freqs_log(Ns, f_s, num=100):
    """
    Return default frequencies vector
    -- Ns:  number of samples in data vector
    -- f_s: sampling frequency
    -- num: number of frequencies required
    """
    T = Ns/f_s
    return np.logspace(np.log10(4/T), np.log10(f_s/2), num=num)


def alias_freq(f, fs):
    if f < 0.5*fs:
        return f
    elif 0.5*fs < f < fs:
        return fs - f
    else:
        return alias_freq(f%fs, fs)

def in_subplots(seq):
    pl.figure();
    N = len(seq)
    for k,s in enumerate(seq):
        subplot(N,1,k+1);
        plot(s)

def ifnot(a, b):
    "if a is not None, return a, else return b"
    if a == None: return b
    else: return a


def DFoSD(vec, normL=None, th = 1e-6):
    "Remove mean and normalize to S.D."
    normL = ifnot(normL, len(vec))
    m, x  = np.mean, vec[:normL]
    sdx = np.std(x,0)
    out =  np.zeros(vec.shape, vec.dtype)
    if sdx.shape is ():
	if np.abs(sdx) > th:
		out = (vec-m(x))/sdx
    else:
	zi = np.where(np.abs(sdx) < th)[0]
	sdx[zi] = 1.0
	out = (vec-m(x))/sdx
	out[zi]=0
    return out

def confidence_contour(esurf, extent, ax, L=3.0):
    # Show 95% confidence level (against white noise, v=3 \sigma^2)
    ax.contour(esurf, [L], extent=extent,
               cmap=mpl.cm.gray)

def cone_infl(freqs, extent, wavelet, ax):
    try:
        ax.fill_betweenx(freqs,
                         extent[0],
                         extent[0]+wavelet.cone(freqs),
                         alpha=0.5, color='black')
        ax.fill_betweenx(freqs,
                         extent[1]+wavelet.cone(-freqs),
                         extent[1],
                         alpha=0.5, color='black')
    except:
        print("Can't use fill_betweenx function: update\
        maptlotlib?")


def wavelet_specgram(signal, f_s, freqs,  ax,
                     wavelet = pycwt.Morlet(),
                     padding = 'mpd',
                     cax = None,
                     vmin=None, vmax=None,
                     correct = None,
		     cwt_fn = pycwt.eds,
                     confidence_level = False,
		     title = "",
		     cmap=swanrgb):
    wcoefs = pycwt.cwt_f(signal, freqs, f_s, wavelet, padding)
    surf = cwt_fn(wcoefs, wavelet.f0)
    
    if correct == 'freq1':
        coefs = freqs*2.0/np.pi
        for i in xrange(surf.shape[1]):
            surf[:,i] *= coefs

    if vmax is None: vmax = np.percentile(surf, 99.5)
    if vmin is None: vmin = np.percentile(surf, 0.5)


    endtime = len(signal)/f_s
    extent=[0, endtime, freqs[0], freqs[-1]]
    im = ax.imshow(surf, extent = extent,
                   origin = 'low',
                   aspect='auto',
                   vmin = vmin, vmax = vmax,
                   cmap = cmap,
                   alpha = 0.95)
    if not cax:
        pl.colorbar(im, ax=ax)
    else:
        pl.colorbar(im, cax = cax)
    cone_infl(freqs, extent, wavelet, ax)
    if confidence_level:
        confidence_contour(eds, extent, ax, confidence_level)
    return surf

def setup_axes_for_spectrogram(figsize = (12,6)):
    "Set up axes for a plot with signal, spectrogram and a colorbar"
    fig = pl.figure(figsize = figsize)
    ax = [fig.add_axes((0.08, 0.4, 0.8, 0.5))]
    ax.append(fig.add_axes((0.08, 0.07, 0.8, 0.3), sharex=ax[0]))
    ax.append(fig.add_axes((0.9, 0.4, 0.02, 0.5), 
                           xticklabels=[], 
                           yticklabels=[]))
    return fig,ax


def plot_wavelet_coherence_with_ts(s1, s2, f_s, freqs=None,
                                   sc_scale='log',
				   figsize=(12,6),
				   cmap='spectral'):
    l1,l2 = map(len, (s1,s2))
    if l1 != l2:
	raise ValueError("two input signals must have same length")
    Ns = float(len(s1))
    if freqs is None:
        if sc_scale == 'log':
            freqs = default_freqs_log(Ns, f_s,512)
        elif sc_scale == 'lin':
            freqs = default_freqs(Ns, f_s,512)

    tvec = np.arange(0, (Ns+2)/f_s, 1./f_s)[:Ns]
    fig,axlist = setup_axes_for_spectrogram(figsize)
    fig2,axlist2 = setup_axes_for_spectrogram(figsize)    

    axlist[1].plot(tvec, s1,'r-',lw=0.5)
    axlist[1].plot(tvec, s2,'b-',lw=0.5)

    axlist2[1].plot(tvec, s1,'r-',lw=0.5)
    axlist2[1].plot(tvec, s2,'b-',lw=0.5)


    surf,phase = pycwt.wtc_f(s1,s2,freqs,f_s, sc_scale=sc_scale)
    if sc_scale is 'log':
        extent = (tvec[0], tvec[-1], np.log(freqs[0]), np.log(freqs[-1]))
    else:
        extent = (tvec[0], tvec[-1], freqs[0], freqs[-1])

    im = axlist[0].imshow(surf, extent=extent,cmap=cmap)
    pl.colorbar(im, cax = axlist[2])
    axlist[0].axis(extent)
    
    im2 = axlist2[0].imshow(phase/np.pi, extent=extent,cmap='jet')    
    pl.colorbar(im2, cax = axlist2[2])
    axlist2[0].axis(extent)

    if sc_scale =='log':
        nticks = 5;
        yticks = np.logspace((np.round(np.log10(freqs[0]))),
                             (np.round(np.log10(freqs[-1]))),nticks)
        yticklabels = ['%2.2f'%i for i in yticks]
        axlist[1].set_xlabel("Time, s")    
        axlist2[1].set_xlabel("Time, s")

        pl.setp(axlist[0], yticks=np.log(yticks), yticklabels=yticklabels)
        pl.setp(axlist2[0], yticks=np.log(yticks), yticklabels=yticklabels)

    pl.setp(axlist[0], ylabel='Frequency, Hz')
    pl.setp(axlist2[0], ylabel='Frequency, Hz')

    fig.suptitle('Wavelet coherence')
    fig2.suptitle('Wavelet phase shift')
    

def plot_spectrogram_with_ts(signal, f_s, freqs=None,
                             figsize=(12,6),
                             lc = 'k', title_string = '',
                             **kwargs):
    "Create a figure of a signal, spectrogram and a colorbar"
    Ns = float(len(signal))
    freqs = ifnot(freqs, default_freqs(Ns, f_s,512))
    tvec = np.arange(0, (Ns+2)/f_s, 1./f_s)[:Ns]

    fig,axlist = setup_axes_for_spectrogram(figsize)

    axlist[1].plot(tvec, signal,'-',color=lc)

    kwargs['cax'] = axlist[2]
    surf = wavelet_specgram(signal, f_s, freqs,  axlist[0], **kwargs)
    axlist[0].set_title(title_string)
    axlist[0].axis((tvec[0],tvec[-1], freqs[0],freqs[-1]))
    #axlist[1].xlim((tvec[0],tvec[-1]))
    return fig, surf




import numpy as np
from scipy.interpolate import splrep,splev

def in_range(v, low, high):
    if low > high:
        low,high = high,low
    return (v > low) * (v <= high)
    

def locextr(x, mode = 1, **kwargs):
   "Finds local maxima when mode = 1, local minima when mode = -1"
   tck = splrep(range(len(x)),x, **kwargs)
   res = 0.05
   xfit = np.arange(0,len(x), res)
   dersign = mode*np.sign(splev(xfit, tck, der=1))
   return xfit[dersign[:-1] - dersign[1:] > 1.5]

def all_vert_maxima(arr):
    return map(locextr, arr)

def limit_bounds(vec, lbound, ubound):
   out = np.copy(vec)
   if lbound:
      out = out[out >= lbound]
   if ubound:
        out = out[out <= ubound]
   return out
         

def locmax_pos(m):
    mfreqs,mtimes = np.where(np.diff(np.sign(np.diff(m, axis=0)), axis=0) < 0)
    return mtimes, mfreqs+2



def trace(seq, seed, memlen = 2, memweight = 0.75,
          lbound = None, ubound = None):
   out = []
   seeds = seed*np.ones(memlen)
   for k,el in enumerate(seq):
      el = limit_bounds(el, lbound, ubound)
      if len(el) > 0:
         j = np.argmin(abs(el - np.mean(seeds)))
         seeds[:-1] = seeds[1:]
         seeds[-1] = el[j]
         out.append((k,el[j]))
   return np.array(out)

def ind_to_val(ind, vrange, nsteps, dv=None):
    if dv is None: dv = (vrange[1] - vrange[0])/float(nsteps)
    return vrange[0] + ind*dv

def val_to_ind(val, vrange, nsteps, dv=None):
    if dv is None: dv = (vrange[1] - vrange[0])/float(nsteps)
    return (val-vrange[0])/dv


def trace_rhythm(arr, seed, extent, start = None, stop=None,rhsd = 0.1,
		 memlen=8):
    """
    traces horizontal ridge in array and returns time, frequency and energy
    vectors
    """
    if start is None:
        start = extent[0]
    if stop is None:
        stop = extent[1]
    
    trange, frange = extent[:2], extent[2:]
    nfreqs, ntimes = arr.shape
    df = (frange[1] - frange[0])/nfreqs
    rhsd_i = rhsd/df


    start_i = val_to_ind(start, trange, ntimes)
    stop_i = val_to_ind(stop, trange, ntimes)
    seed_i = val_to_ind(seed, frange, nfreqs)

    xtimes, xfreqs = locmax_pos(arr[:,start_i:stop_i])
    xu = np.unique(xtimes)

    #arr_tr = map(locextr, arr[:,start_i:stop_i].T)
    arr_tr = (xfreqs[xtimes==i] for i in xu)
    
    rhythm = trace(arr_tr, seed_i,
                   memlen=memlen,
                   lbound = seed_i - 2*rhsd_i,
                   ubound = seed_i + 2*rhsd_i)
    rhythm[:,0] += start_i

    tck = splrep(rhythm[:,0], rhythm[:,1])
    rhfreqs = np.asarray(map(int,
                             np.round(splev(np.arange(start_i, ntimes),
                                            tck))))

    fvec = ind_to_val(rhythm[:,1], frange, nfreqs)
    tvec = ind_to_val(rhythm[:,0], trange, ntimes)
    evec = np.asarray([np.sum(arr[rhfreqs[i]-rhsd:rhfreqs[i]+rhsd,i+start_i])
                       for i in np.arange(len(tvec))])
    return tvec, fvec, evec
                  















# Continuous wavelet transfrom via Fourier transform
# Collection of routines for wavelet transform via FFT algorithm


#-- Some naming and other conventions --
# use f instead of omega wherever rational/possible
# *_ft means Fourier transform

#-- Some references --
# [1] Mallat, S.  A wavelet tour of signal processing
# [2] Addison, Paul S. The illustrated wavelet transform handbook
# [3] Torrence and Compo. A practical guide to wavelet
#     analysis. Bulletin of American Meteorological Society. 1998, 79(1):61-78

import numpy as np
from numpy.fft import fft, ifft, fftfreq



_maxshape_ = 1e8 ## for arrays larger than this number of values use
                 ## memory-mapped arrays

_maxexpanded_ = 5e6 ## only expand up to the next power of 2 signals shorter
                   ## than this

def memsafe_arr(shape, dtype):
    import tempfile as tmpf
    N = shape[0] * shape[1]
    if N < _maxshape_:
        return np.zeros(shape, dtype=dtype)
    else:
        print "Using memory-mapped arrays..."
        _tmpfile = tmpf.TemporaryFile('w+')
        return np.memmap(_tmpfile, dtype=dtype, shape=shape)
        
    

#try:
#    from scipy.special import gamma
#except:

pi = np.pi

try:
    from scipy.special import gamma
    class DOG:
        """Derivative of Gaussian, general form"""
        # Incomplete, as the general form of the mother wavelet
        # would require symbolic differentiation.
        # Should be enough for the CWT computation, though

        def __init__(self, m = 1.):
            self.order = m
            self.fc = (m+.5)**.5 / (2*pi)
            self.f0 = self.fc
    
        def psi_ft(self, f):
            c = 1j**self.order / np.sqrt(gamma(self.order + .5)) #normalization
            w = 2*pi*f
            return c * w**self.order * np.exp(-.5*w**2)
except:
    class DOG:
        def __init__(self,*args):
            print "Couldn't create DOG wavelet because scipy is unavailable"
            return None
        

class Mexican_hat:
    def __init__(self, sigma = 1.0):
        self.sigma = sigma
        self.fc = .5 * 2.5**.5 / pi
        self.f0 = self.fc
    def psi_ft(self, f):
        """Fourier transform of the Mexican hat wavelet"""
        c = np.sqrt(8./3.) * pi**.25 * self.sigma**2.5 
        wsq = (2. * pi * f)**2.
        return -c * wsq * np.exp(-.5 * wsq * self.sigma**2.)
    def psi(self, tau):
        """Mexian hat wavelet as described in [1]"""
        xsq = (tau / self.sigma)**2.
        c = 2 * pi**-.25 / np.sqrt(3 * self.sigma) # normalization constant from [1]
        out =  c * (1 - xsq) * np.exp(-.5 * xsq)
        out *= (tau < 6) * (tau > -6) # make it 'compact support'
        return out
    def cone(self, f):
        "e-folding time [TC98]. For frequencies"
        return self.f0*2.0**0.5 / f
    def cone_s(self, s):
        "e-folding time [TC98]. For scales"
        return 2.**0.5*s
    def set_f0(self, f0):
        pass

def heavyside(x):
    return 1.0*(x>0.0)

class Morlet:
    def __init__(self, f0 = 1.5):
        self.set_f0(f0)
    def psi(self, t):
        return pi**(-0.25) * np.exp(-t**2 / 2.) * np.exp(2j*pi*self.f0*t) #[3]
    def psi_ft(self, f):
        """Fourier transform of the approximate Morlet wavelet
            f0 should be more than 0.8 for this function to be
            correct."""
        # [3]
        #coef = (pi**-.25) * heavyside(f)
        coef = (pi**-.25) * 2**0.5 # (Addison)
        return  coef * np.exp(-.5 * (2 * pi * (f - self.f0))**2.)
    def sigma_t_s(self,s):
        """
        Heisenberg box width for scales [Addison]
        s/sqrt(2)
        """
        return s/np.sqrt(2)

    def sigma_f_s(self, s):
        """
        Heisenberg box height for scales [Addison]
        1/(s*pi*2*sqrt(2))
	// close to simulations of |W|^2 for sine waves
	// with fixed frequencies
        """
        return 1.0/(s*pi*np.sqrt(8.0))

    def sigma_f_f(self,f):
        """
        Heisenberg box height in frequencies
        s*sqrt(2)/4
        """
        return self.sigma_f_s(self.f0/f)

    def cone(self, f):
        "e-folding time [Addison]. For frequencies"
        return self.f0/(f*2.0**0.5)
    def cone_s(self, s):
        "e-folding time [Addison]. For scales"
        return self.sigma_t_s(s)
    def set_f0(self, f0):
        self.f0 = f0
        #self.fc = f0


def next_pow2(x):
    return 2.**np.ceil(np.log2(x))

def pad_func(ppd):
    func = None
    if ppd == 'zpd':
        func = lambda x,a:  x*0.0
    elif ppd == 'cpd':
        func = lambda x,a:  a*np.ones(np.size(x))
    elif ppd == 'mpd':
	func = lambda x,a: x[::-1]
    return func
    
def evenp(n):
    return not n%2

def cwt_a(signal, scales, sampling_scale = 1.0,
          wavelet=Mexican_hat(),
          ppd = 'mpd',
          verbose = False,):
    """ Continuous wavelet transform via fft. Scales version."""
    import sys
    siglen = len(signal)

    if hasattr(wavelet, 'cone_s'):
        needpad = wavelet.cone_s(np.max(scales))/sampling_scale
    else:
        needpad = 1.2*np.max(scales)
    needpad = int(round(needpad))

    if siglen < _maxexpanded_: # only do expansion up to next ^2 for short signals
        ftlen = int(next_pow2(siglen + 2*needpad))
    else:
        if not evenp(siglen) :
            siglen -= 1
            signal = signal[:-1]
        ftlen = int(siglen + 2*needpad)

    padlen1 = int(round((ftlen - siglen)/2))
    padlen2 = evenp(siglen) and padlen1 or padlen1+1
    
    padded_signal = np.ones(ftlen)
    padfunc = pad_func(ppd)
    padded_signal[:padlen1] = padfunc(padded_signal[:padlen1],signal[0])
    padded_signal[-padlen2:] = padfunc(padded_signal[-padlen2:],signal[-1])

    padded_signal[padlen1:-padlen2] = signal



    signal_ft = fft(padded_signal)     # FFT of the signal
    del padded_signal
    ftfreqs = fftfreq(ftlen, sampling_scale)  # FFT frequencies

    psi_ft = wavelet.psi_ft
    coef = np.sqrt(2*pi/sampling_scale)

    ### version with map is slower :(
    #def _ls(s):
    #    return ifft(signal_ft*coef*(s**0.5)*np.conjugate(psi_ft(s*ftfreqs)))[padlen:-padlen]
    #xx = map(_ls, scales)
    #W = np.array(xx)

    ### matrix version
    scales = scales.reshape(-1,1)
    #s_x_ftf = np.dot(scales, ftfreqs.reshape(1,-1))
    #psi_ft_bar = np.conjugate(psi_ft(s_x_ftf))
    #psi_ft_bar *= coef*np.sqrt(scales)
    #W = ifft( psi_ft_bar * signal_ft.reshape(1,-1))[:,padlen1:-padlen2]
    #return W

    ## create the result matrix beforehand
    #W = np.zeros((len(scales), siglen), 'complex')
    W = memsafe_arr((len(scales), siglen), 'complex')
    ## Now fill in the matrix
    for n,s in enumerate(scales):
        if verbose :
            sys.stderr.write('\r Processing scale %04d of %04d'%(n+1,len(scales)))
        psi_ft_bar = np.conjugate(psi_ft(s * ftfreqs))
        psi_ft_bar *= coef*np.sqrt(s) # Normalization from [3]
        W[n,:] = ifft(signal_ft * psi_ft_bar)[padlen1:-padlen2]
    return W


def cwt_f(signal, freqs, Fs=1.5, wavelet = Morlet(), ppd = 'zpd', verbose=False):
    """Continuous wavelet transform -- frequencies version"""
    scales = wavelet.f0/freqs
    dt = 1./Fs
    return cwt_a(signal, scales, dt, wavelet, ppd, verbose=verbose)


def eds(x, f0=1.5):
    "Energy density surface [2,3]"
    ## Update, simulations with MK (as in [3]) suggest that I
    ## shouldn't divide by f0 to obtain correct normalisation,
    ## e.g. 1 s.d. is mean for white noise signal
    #return abs(x)**2/f0
    return abs(x)**2

def real(x, *args):
    return x.real

def cwt_phase(x, *args):
    return np.arctan2(x.imag, x.real)


def xwt_f(sig1, sig2, freqs, Fs=1.0, wavelet=Morlet()):
    "Cross-wavelet coeficients for 2 signals"
    cwtf = lambda x: cwt_f(x, freqs, Fs, wavelet)
    return xwt(cwtf(sig1),cwtf(sig2))

def absxwt_f(sig1, sig2, freqs, Fs=1.0, wavelet=Morlet()):
    "Cross-wavelet power for 2 signals"
    return abs(xwt_f(sig1,sig2, freqs, Fs, wavelet))/wavelet.f0**0.5

def absxwt_a(sig1, sig2, scales, dt=1.0, wavelet=Morlet()):
    "Cross-wavelet power for 2 signals"
    Fs = 1./dt
    freqs = wavelet.f0/scales
    return abs(xwt_f(sig1,sig2, freqs, Fs, wavelet))/wavelet.f0**0.5


def xwt(wcoefs1,wcoefs2):
    "Cross wavelet transform for 2 sets of coefficients"
    return wcoefs1*wcoefs2.conjugate()

def absxwt(wcoefs1,wcoefs2, f0=1.0):
    "Cross-wavelet power for 2 sets of coefficients"
    ## Why do I divide by f_0^2 here?
    return  abs(xwt(wcoefs1,wcoefs2))/f0**0.5

def wtc_a(sig1, sig2, scales, dt=1.0, wavelet=Morlet(1.0),sc_scale='log'):
    cwta = lambda x: cwt_a(x, scales, dt, wavelet)
    return coherence_a(cwta(sig1), cwta(sig2), scales, dt, wavelet.f0, sc_scale=sc_scale)


def wtc_f(sig1, sig2, freqs, Fs=1.0, wavelet=Morlet(f0=1.0),
          sc_scale='log'):
    cwtf = lambda x: cwt_f(x, freqs, Fs, wavelet)
    return coherence_f(cwtf(sig1), cwtf(sig2), freqs, Fs, wavelet.f0,sc_scale)

def coherence_f(x,y,freqs, Fs=1.0, f0=1.0, sc_scale = 'log'):
    return coherence_a(x,y,f0/freqs, 1.0/Fs, f0, sc_scale)

def coherence_a(x,y,scales,dt,f0=1.0,sc_scale = 'log'):
    # almost not useful 
    scv = np.reshape(scales, (-1,1))
    sx = wsmooth_a((abs(x)**2)/scv,scales,dt=dt,f0=f0,sc_scale=sc_scale)
    sy= wsmooth_a((abs(y)**2)/scv,scales,dt=dt,f0=f0,sc_scale=sc_scale)
    sxy = wsmooth_a((x*y.conjugate())/scv, scales, dt=dt,f0=f0,sc_scale=sc_scale)
    out_ph = np.arctan2(sxy.real, sxy.imag)
    out =  abs(sxy)**2/(sx.real*sy.real)
    out = np.ma.masked_invalid(out)
    
    return out, np.ma.masked_invalid(out_ph)


def cphase(x,y):
    #hard to interprete :(
    d = xwt(x,y)
    return np.arctan2(d.imag, d.real)

def wsmooth_a_conv(coefs, scales, dt=1.0, f0=1.0,
                   sc_scale ='log',
                   dj0=0.6):
    "Smoothing of wavelet coefs. Scales version, scales must be in log scale (octaves)"
    from scipy import signal
    G = lambda t,s: np.exp(-0.5*(t/s)**2) # Torrence and Webster 1998
    # G = lambda t,s : (1/s*(2*pi)**0.5)*exp(-(t/2)**2) ## Bloomfield et al 2004

    # "correct Ghat for G"
    #Ghat = lambda k,s: s*((2*pi)**0.5) * np.exp(-pi**2 * k**2 * 2 * s**2)
    Ghat = lambda k,s: s*np.exp(-0.5*(s*2*pi*k)**2)


    W = np.zeros(coefs.shape, 'complex')
    fftk = fftfreq(coefs.shape[1], dt)

    for n,s in enumerate(scales):
	kernhat = Ghat(fftk, s)
	W[n,:] = ifft(fft(coefs[n,:]) * kernhat)

    if sc_scale == 'log':
        dj0steps = dj0/(2*abs(np.mean(np.diff(np.log2(scales)))))
        kern = np.concatenate([[dj0steps%1], np.ones(2*np.round(dj0steps)-1),
                               [dj0steps%1]])
        kern /= np.sum(kern)

        for j in xrange(coefs.shape[1]):
            W[:,j] = np.convolve(W[:,j], kern,mode='same')

    else:
        print "not smoothing in scales, as scales are not in log stepping"
    return W

def coherence_a_diff(W1,W2,scales,dt,f0=1.0,h=0.1, tdiffuse=10,
		     acc_sp = 10,
		     *args,**kwargs):
    "wavelet coherence with smoothing via diffusion"
    scv = np.reshape(scales, (-1,1))
    sx_p = (abs(W1)**2)/scv
    sy_p = (abs(W2)**2)/scv
    sxy_p = (W1*W2.conjugate())/scv
    wcoher = lambda sx,sy,sxy: abs(sxy)**2/(sx.real*sy.real)
    acc = [wcoher(sx_p, sy_p, sxy_p)]
    #t = np.linspace(0,tdiffuse, acc_sp)
    #sx_n = _wsmooth_a_diffusion_integrate(sx_p, t, scales, dt, *args,**kwargs)
    #sy_n = _wsmooth_a_diffusion_integrate(sy_p, t, scales, dt, *args,**kwargs)
    #sxy_n = _wsmooth_a_diffusion_integrate(sxy_p, t, scales, dt, *args,**kwargs)
    #acc  = [wcoher(ax,ay,axy) for ax,ay,axy in zip(sx_n, sy_n, sxy_n)]
    for k,te in enumerate(arange(0,tdiffuse,h)):
     	sx_n = sx_p + h*_wsmooth_a_diffusion_tderiv(sx_p, scales,dt,*args,**kwargs)
     	sy_n = sy_p + h*_wsmooth_a_diffusion_tderiv(sy_p,scales,dt,*args,**kwargs)
     	sxy_n  = sxy_p + h*_wsmooth_a_diffusion_tderiv(sxy_p,scales,dt, *args,**kwargs)
     	if not k%acc_sp:
     	    acc.append(wcoher(sx_n, sy_n, sxy_n))
     	sx_p, sy_p, sxy_p = sx_n, sy_n, sxy_n
     	sys.stderr.write('\r diff time: %03.3f'%te)
    acc.append(wcoher(sx_n, sy_n, sxy_n))
    return acc


def _wsmooth_a_diffusion_integrate(I0, t, *args):
    from scipy import integrate
    sh = I0.shape
    def _step(y,t,*args):
	im = y.reshape(sh)
	return _wsmooth_a_diffusion_tderiv(im, *args).reshape(-1)
    y0 = I0.reshape(-1)
    I2 = integrate.odeint(_step,y0,t,args)
    return np.array([i2.reshape(sh) for i2 in I2])


def _wsmooth_a_diffusion_tderiv(I, scales, dt, psi=Morlet(),pow=1.):
    beta = 1/(2*pi*2**0.5)
    Dx  = np.reshape(psi.sigma_t_s(scales)**pow, (-1,1))
    Ds = np.reshape(psi.sigma_f_s(scales)**pow, (-1,1))
    kdiff = np.array([1,-2,1])
    Diff_x = np.zeros(I.shape, 'complex')
    Diff_s = np.zeros(I.shape, 'complex')
    Diff_x = signal.convolve2d(I, kdiff.reshape(1,-1), mode='same')
    Diff_s = signal.convolve2d(I, kdiff.reshape(-1,1), mode='same')    
    ## for n in xrange(len(scales)):
    ##     Diff_x[n,:] = np.convolve(I[n,:], kdiff,mode='same')
    ## for j in xrange(I.shape[1]):
    ##     Diff_s[:,j] = np.convolve(I[:,j], kdiff, mode='same')
    return Dx*Diff_x + Ds*Diff_s



def wsmooth_a_diffusion(coefs, scales, dt=0.1, f0=1.0, sc_scale='lin',
                        alpha=1.0e-2, niter=1600):
    from scipy import signal
    beta = 1/(2*pi*2**0.5)

    Dx = (scales/2**0.5).reshape(-1,1)
    Ds = (beta/scales).reshape(-1,1)

    Wprev = coefs
    kdiff = np.array([1,-2,1])
    lapl = np.array([(0,1,0), (1,-4,1), (0,1,0)])
    #acc = [coefs]
    for k in xrange(niter):
        if not k%10:
            sys.stderr.write('\r iteration: %0d'%k)
	Diff_x = np.zeros(coefs.shape, 'complex')
	Diff_s = np.zeros(coefs.shape, 'complex')
	for n in xrange(len(scales)):
	    Diff_x[n,:] = np.convolve(Wprev[n,:], kdiff,mode='same')
	for j in xrange(coefs.shape[1]):
	    Diff_s[:,j] = np.convolve(Wprev[:,j], kdiff, mode='same')
	Diff_x *= Dx
	Diff_s *= Ds
	Wnext = Wprev + alpha*(Diff_x + Diff_s)
	d = np.std(Wnext-Wprev)
	#acc.append(Wnext.real)
	Wprev = Wnext.copy()
    return Wnext

wsmooth_a = wsmooth_a_conv



def mirrorpd(k, L):
    if 0 <= k < L : return k
    else: return -(k)%L


def phase_coherence_other(sig1,sig2,freqs, Fs=1.0, wavelet=Morlet()):
    _w1 = cwt_phase(cwt_f(sig1, freqs, Fs, wavelet))
    _w2 = cwt_phase(cwt_f(sig2, freqs, Fs, wavelet))
    L = len(sig1)
    out = np.zeros(_w1.shape)
    phdiff = np.exp(1j*(_w1 - _w2))
    ns = 10
    for j in range(len(freqs)-ns):
        if hasattr(wavelet, 'cone_s'):
            needpad = wavelet.cone_s(wavelet.f0/freqs[j])*Fs
        else:
            needpad = 1.2*np.max(wavelet.f0/freqs)*Fs

        padlen = int(np.round(needpad))
        
        indices = map(lambda i: mirrorpd(i, L),
                      range(-padlen, 0) + range(0,L) + range(L, L+padlen))
        s = phdiff[j:j+ns,indices]
        for i in range(L):
            out[j,i] = np.abs(np.mean(s[:,i:i+2*padlen]))
    return out, np.abs(phdiff.mean(axis=1))

import time 
def speed_test(sig_len, nscales = 512, N = 100, ppd='zpd',wavelet=Morlet()):
    tick = time.clock()
    s = np.random.randn(sig_len)
    scales = np.linspace(0.5, sig_len/4.0, nscales)
    for j in xrange(N):
        eds = cwt_a(s, scales, sampling_scale = 1.0, ppd = ppd,
                    wavelet=wavelet)
    tock =  time.clock() - tick
    print tock
    return tock

